﻿using System;
using System.Linq;

namespace _2011_04_12_2011_04_13_Praktikum
{
    class DataSeries
    {
        protected string title;
        protected double[] xValues, yValues;
        protected double xMin, xMax, yMin, yMax;

        public DataSeries()
        {
            title = "Leer";
            xValues = new double[] {0.0, 1.0};
            yValues = new double[] {0.0, 1.0};
            xMin = 0.0;
            xMax = 1.0;
            yMin = 0.0;
            yMax = 1.0;
        }

        public DataSeries(string path)
        {
            // Todo: Hier sollten ein paar Exceptions gefangen werden; das war aber noch nicht dran.
            string[] lines = System.IO.File.ReadAllLines(path);
            title = lines[0];
            xValues = new double[lines.Length-1];
            yValues = new double[lines.Length-1];
            for (int i = 1; i < lines.Length; i++)
			{
			    string[] parts = lines[i].Split(';');
                xValues[i-1] = double.Parse(parts[0]);
                yValues[i-1] = double.Parse(parts[1]);
			}
            xMin = xValues.Min();
            xMax = xValues.Max();
            yMin = yValues.Min();
            yMax = yValues.Max();
        }

        public override string ToString()
        {
            return title;
        }

        public double XMin
        {
            get { return xMin; }
        }

        public double XMax
        {
            get { return xMax; }
        }

        public double YMin
        {
            get { return yMin; }
        }

        public double YMax
        {
            get { return yMax; }
        }

        public virtual System.Windows.Shapes.Polyline BuildPolyline(double width, double height)
        {
            // Todo: Max könnte gleich Min sein!
            double scaleX = width / (XMax - XMin);
            double scaleY = height / (YMax - YMin);
            System.Windows.Shapes.Polyline polyline = new System.Windows.Shapes.Polyline();
            polyline.StrokeThickness = 2.0;
            polyline.Stroke = System.Windows.Media.Brushes.Red;
            for (int i = 0; i < xValues.Length; i++)
			{
                // Die y-Achse zeigt auf dem Bildschirm nach unten!
			    polyline.Points.Add(new System.Windows.Point(scaleX*(xValues[i]-XMin), scaleY*(YMax-yValues[i])));
			}
            return polyline;
        }
    }

    class SineWave : DataSeries
    {
        public SineWave(double amplitude, double frequency, double initialPhase, double xMin, double xMax)
        {
            title = "Sinus " + frequency;
            int n = (int)(42.0 * frequency * (xMax - xMin));
            xValues = new double[n];
            yValues = new double[n];
            for (int i = 0; i < n; i++)
            {
                xValues[i] = xMin + (xMax-xMin)*i/(n-1.0);
                yValues[i] = amplitude * Math.Sin(2.0 * Math.PI * frequency * xValues[i] + initialPhase);
            }
            this.xMin = xMin;
            this.xMax = xMax;
            this.yMin = -amplitude;
            this.yMax = amplitude;
        }

        public override System.Windows.Shapes.Polyline BuildPolyline(double width, double height)
        {
            // Todo: Nicht den Code von DataSeries hier verdoppeln
            // Todo: Max könnte gleich Min sein!
            double scaleX = width / (XMax - XMin);
            double scaleY = height / (YMax - YMin);
            System.Windows.Shapes.Polyline polyline = new System.Windows.Shapes.Polyline();
            polyline.StrokeThickness = 2.0;
            System.Windows.Media.DoubleCollection c = new System.Windows.Media.DoubleCollection();
            c.Add(6.0);
            c.Add(2.0);
            polyline.StrokeDashArray = c;
            polyline.Stroke = System.Windows.Media.Brushes.Blue;
            for (int i = 0; i < xValues.Length; i++)
            {
                // Die y-Achse zeigt auf dem Bildschirm nach unten!
                polyline.Points.Add(new System.Windows.Point(scaleX * (xValues[i] - XMin), scaleY * (YMax - yValues[i])));
            }
            return polyline;
        }
    }
}
